#!/bin/sh
#
# This script is used to install device driver module on linux for HighPoint product
#
# Copyright (C) 2013 HighPoint Technologies, Inc.
# All Rights Reserved.
#

product=hptnvme
module=hptnvme.ko
targetname=hptnvme

export PATH=/sbin:/usr/sbin:$PATH

if ( which mkinitrd > /dev/null 2>&1 ); then
  continue
elif ( which dracut > /dev/null 2>&1 ); then
  continue
else
  echo "No mkinitrd or dracut exists. Exit now."
  exit 1
fi

MODULE=`echo $module | cut -d. -f1`
CWD=`pwd`

if [ -f modules.cgz ]; then
	echo "Start to install the driver $module of $product."
	cd /lib/modules/
	for dir in [23].*; do
		[ -d $dir/kernel/fs ] || continue
		mod=`find $dir/kernel/fs -type f -name *.ko.xz | head -1`
		vermagic=`modinfo -F vermagic $mod`
		arch=`uname -m`
		( echo "$vermagic" | grep " 586 " -s -q ) && arch=i586
		( echo "$vermagic" | grep " 686 " -s -q ) && arch=i686
		( echo "$vermagic" | grep " x86_64 " -s -q ) && arch=x86_64
		# echo $mod $vermagic $arch
		[ -f $dir/modules.dep ] || continue
		if [ -f $dir/kernel/drivers/scsi/${module} -o -f $dir/kernel/drivers/scsi/${targetname}/${module} -o -f $dir/kernel/updates/${module} ]; then
			echo -n "The driver had been installed to the system. Do you want to replace with the new one?(Y/n): "
			read ans
			if [ "${ans}" != "" -a "${ans}" != "Y" -a "${ans}" != "y" ]; then
				continue
			fi
			find -type f -name ${module} -exec rm -f {} \;
		fi
		gzip -dc $CWD/modules.cgz | cpio -idumv $dir/$arch/$module >/dev/null 2>&1
		if [ -f $dir/$arch/$module ]; then
			echo "Install the driver for $dir."
			mkdir -p $dir/kernel/drivers/scsi
			mv $dir/$arch/$module $dir/kernel/drivers/scsi/
			rmdir $dir/$arch
			# remove conflicted driver module, TODO use modinfo to determine pci_id
			if [ "${MODULE}" = "hptmv" -o "${MODULE}" = "hptmv6"  -o "${MODULE}" = "rr172x" -o "${MODULE}" = "rr174x" -o "${MODULE}" = "rr2310_00" ] ; then
				find $dir -name sata_mv.* -exec rm -f {} \;
			elif [ "$MODULE" = "hpt374" -o "$MODULE" = "hpt37x2" ]; then
				find $dir -name pata_hpt37x.* -exec rm -f {} \;
			elif [ "${MODULE}" = "rr272x_1x" -o "${MODULE}" = "rr274x_3x" -o  "${MODULE}" = "rr276x" -o "${MODULE}" = "rr278x" ]; then
				find $dir -name mvsas.* -exec rm -f {} \;
			fi
			depmod -a $dir
		else
			echo "Fail to install $dir/kernel/drivers/scsi/$module."
			continue
		fi
		if [ -f /boot/initrd-$dir.img ] ; then
			gzip -dc /boot/initrd-$dir.img > /tmp/${MODULE}loop.img 2>/dev/null
			if [ "$?" != 0 ] ; then
				continue
			fi
			mkdir -p /tmp/${MODULE}loop
			mount -o loop /tmp/${MODULE}loop.img /tmp/${MODULE}loop >/dev/null 2>/dev/null
			if [ "$?" != 0 ] ; then
				if ( cpio -itv < /tmp/${MODULE}loop.img 2> /dev/null | grep -s -q ${module} ); then
					INTRD=1
					( grep -s -q scsi_mod /lib/modules/$dir/modules.dep ) && WITHSCSI="--preload scsi_mod"
					( grep -s -q sd_mod /lib/modules/$dir/modules.dep ) && WITHSD="--preload sd_mod"
					( grep -s -q scsi_wait_scan /lib/modules/$dir/modules.dep ) && WITHWAIT="--with=scsi_wait_scan"
					echo "Backup /boot/initrd-$dir.img to /boot/initrd-$dir.img.${MODULE}."
					mv /boot/initrd-$dir.img /boot/initrd-$dir.img.${MODULE}
					mkinitrd ${WITHSCSI} ${WITHSD} --with=${MODULE} ${WITHWAIT} /boot/initrd-$dir.img $dir
				fi
			else
				HPTMV6=$( find /tmp/${MODULE}loop/* -name ${module} )
				if test "${HPTMV6}" != "" ; then
					INTRD=1
					cp -f /lib/modules/$dir/kernel/drivers/scsi/${module} ${HPTMV6}
					umount /tmp/${MODULE}loop.img
					echo "Backup /boot/initrd-$dir.img to /boot/initrd-$dir.img.${MODULE}."
					mv /boot/initrd-$dir.img /boot/initrd-$dir.img.${MODVER}
					gzip -9c /tmp/${MODULE}loop.img > /boot/initrd-$dir.img
				else
					umount /tmp/${MODULE}loop.img
				fi
			fi
		elif [ -f /boot/initramfs-$dir.img ] ; then
			gzip -dc /boot/initramfs-$dir.img > /tmp/${MODULE}loop.img 2>/dev/null
			if [ "$?" != 0 ] ; then
				continue
			fi
			if ( cpio -itv < /tmp/${MODULE}loop.img 2> /dev/null | grep -s -q ${module} ); then
				INTRD=1
				echo "Backup /boot/initramfs-$dir.img to /boot/initramfs-$dir.img.${MODULE}."
				mv /boot/initramfs-$dir.img /boot/initramfs-$dir.img.${MODULE}
				dracut --add-drivers ${MODULE} /boot/initramfs-$dir.img $dir
			fi
		fi
		rm -rf /tmp/${MODULE}loop /tmp/${MODULE}loop.img
	done
	echo "Finish installing the kernel module $module."
	if [ "$INTRD" == "1" ]; then
		echo "Please reboot the system to use the new driver module."
		exit 0
	elif [ ! -f /etc/rc.modules ] || ( ! grep $MODULE /etc/rc.modules -s -q ); then
		echo -e /sbin/modprobe sd_mod\\n/sbin/modprobe $MODULE >> /etc/rc.modules
		chmod 755 /etc/rc.modules
	fi
	if [ "${MODULE}" = "hptmv" -o "${MODULE}" = "hptmv6" -o "${MODULE}" = "rr172x" -o "${MODULE}" = "rr174x" -o "${MODULE}" = "rr2310_00" ] ; then
		# centos 5.2 add sata_mv in initrd
		( cd /lib/modules/
		for dir in [23].*; do
			[ -d $dir/kernel/fs ] || continue
			if [ -f /boot/initrd-$dir.img ]; then
				mv /boot/initrd-$dir.img /boot/initrd-$dir.img.bak
				mkinitrd --builtin=sata_mv /boot/initrd-$dir.img $dir
			elif [ -f /boot/initramfs-$dir.img ]; then
				mv /boot/initramfs-$dir.img /boot/initramfs-$dir.img.bak
				dracut /boot/initramfs-$dir.img $dir
			fi
		done
		)
		if ( lsmod | grep sata_mv -s -q ); then
			echo "Unloading conflicting module sata_mv..."
			rmmod sata_mv
			if ( lsmod | grep sata_mv -s -q ) ; then
				echo "Failed to unload module sata_mv, it may be in use, please reboot the system."
				exit 0
			fi
		fi
	elif [ "$MODULE" = "hpt374" -o "$MODULE" = "hpt37x2" ]; then
		( cd /lib/modules/
		for dir in [23].*; do
			[ -d $dir/kernel/fs ] || continue
			if [ -f /boot/initrd-$dir.img ]; then
				mv /boot/initrd-$dir.img /boot/initrd-$dir.img.bak
				# old release not include hpt366 in initrd(? hope so)
				mkinitrd --builtin=pata_hpt37x /boot/initrd-$dir.img $dir
			elif [ -f /boot/initramfs-$dir.img ]; then
				mv /boot/initramfs-$dir.img /boot/initramfs-$dir.img.bak
				dracut /boot/initramfs-$dir.img $dir
			fi
		done
		)
		if ( lsmod | grep hpt366 -s -q ); then
			echo "Unloading conflicting module hpt366..."
			rmmod hpt366
			if ( lsmod | grep hpt366 -s -q ) ; then
				echo "Failed to unload module hpt366, it may be in use, please reboot the system."
				exit 0
			fi
		fi
		if ( lsmod | grep pata_hpt37x -s -q ); then
			echo "Unloading conflicting module pata_hpt37x..."
			rmmod pata_hpt37x
			if ( lsmod | grep pata_hpt37x -s -q ) ; then
				echo "Failed to unload module pata_hpt37x, it may be in use, please reboot the system."
				exit 0
			fi
		fi
	elif [ "${MODULE}" = "hptiop" ] ; then
		( cd /lib/modules/
		for dir in [23].*; do
			[ -d $dir/kernel/fs ] || continue
			if [ -f /boot/initrd-$dir.img ]; then
				mv /boot/initrd-$dir.img /boot/initrd-$dir.img.bak
				# do not include hptiop in initrd, as the udev and so will load the driver
				mkinitrd --builtin=hptiop /boot/initrd-$dir.img $dir
			elif [ -f /boot/initramfs-$dir.img ]; then
				mv /boot/initramfs-$dir.img /boot/initramfs-$dir.img.bak
				# do not include hptiop in initramfs, as the udev and so will load the driver
				dracut /boot/initramfs-$dir.img $dir
			fi
		done
		)
		if ( lsmod | grep hptiop -s -q ); then
			echo "Unloading previous loading module hptiop..."
			rmmod hptiop
			if ( lsmod | grep hptiop -s -q ) ; then
				echo "Failed to unload module hptiop, it may be in use, please reboot the system."
				exit 0
			fi
		fi
	elif [ "${MODULE}" = "rr272x_1x" -o "${MODULE}" = "rr274x_3x" -o  "${MODULE}" = "rr276x" -o "${MODULE}" = "rr278x" ] ; then
		( cd /lib/modules/
		for dir in [23].*; do
			[ -d $dir/kernel/fs ] || continue
			if [ -f /boot/initrd-$dir.img ]; then
				mv /boot/initrd-$dir.img /boot/initrd-$dir.img.bak
				# do not include mvsas in initrd, as the udev and so will load the driver
				mkinitrd --builtin=mvsas /boot/initrd-$dir.img $dir
			elif [ -f /boot/initramfs-$dir.img ]; then
				mv /boot/initramfs-$dir.img /boot/initramfs-$dir.img.bak
				# do not include mvsas in initramfs, as the udev and so will load the driver
				dracut /boot/initramfs-$dir.img $dir
			fi
		done
		)
		if ( lsmod | grep mvsas -s -q ); then
			echo "Unloading previous loading module mvsas..."
			rmmod mvsas
			if ( lsmod | grep mvsas -s -q ) ; then
				echo "Failed to unload module mvsas, it may be in use, please reboot the system."
				exit 0
			fi
		fi
	fi
	echo "Loading $product driver module $MODULE."
	modprobe $MODULE
	exit 0
else
	echo "Usage:"
	echo "# ./install.sh"
fi
exit 1
