#!/bin/sh

DRIVER=hptnvme

echo "Driver Installation"
CWD=`dirname $0 2> /dev/null`
[ "$CWD" == "" ] && CWD=${0%/*}

MACHINE=`uname -m 2> /dev/null`
[ "$MACHINE" == "" ] && MACHINE=i686

KERNEL=`uname -r 2> /dev/null`
[ "$KERNEL" == "" ] && KERNEL=`awk '{ print $3 }' /proc/version 2> /dev/null`
[ "$KERNEL" == "" ] && KERNEL=`cut -d\   -f 3 /proc/version 2> /dev/null`

( echo $KERNEL | grep "386" -s -q ) && MACHINE=i386
( echo $KERNEL | grep "586" -s -q ) && MACHINE=i586
( echo $KERNEL | grep "686" -s -q ) && MACHINE=i686
( echo $KERNEL | grep "x86_64" -s -q ) && MACHINE=x86_64

cd /tmp
if [ -f $CWD/modules.cgz ]; then
  gzip -dc $CWD/modules.cgz | cpio -idum ${KERNEL}/${MACHINE}/${DRIVER}.ko > /dev/null 2>/dev/null
  if [ ! -f ${KERNEL}/${MACHINE}/${DRIVER}.ko ]; then
    echo "Error to extract driver."
    exit 1
  fi
else
  echo "No driver archive found."
  exit 1
fi

if ! cp $CWD/60-persistent-storage-hptblock.rules /etc/udev/rules.d/ ; then
  echo "ERROR! Failed to install udev rule for hptblock."
  exit 1
fi

udevadm control --reload
sleep 1
udevadm trigger
sleep 1

modprobe sd_mod > /dev/null 2> /dev/null
insmod ${KERNEL}/${MACHINE}/${DRIVER}.ko

# refresh device storage
if [ -f /bin/restart-anaconda ]; then
  echo "Notify installer storage changes."
  /bin/restart-anaconda
fi

echo "Driver installation step 1 completed."
