#!/bin/sh

DRIVER=hptnvme

echo "Driver Installation"

MACHINE=`uname -m`
KERNEL=`uname -r`
CWD=`dirname $0`
REMOVE_MOD=NO_CONFLICT
KMOD=ko

FIRSTK=`uname -r | cut -d. -f1`
if [ ${FIRSTK} = 4 ]; then
        BIN=bin
else
	BIN=sbin
fi

# From Fedora 12 change the name to initramfs
# so we should test the release.
INITRD=initrd
# It may be best to check dracut to determine initramfs

[ -x /${BIN}/dracut ] && INITRD=initramfs

( echo $KERNEL | grep "^2.4" -s -q ) && { KMOD=o; }

export PATH=/bin:/sbin:/usr/bin:/usr/sbin

if  [ ${DRIVER} = "hptiop" ]; then
	REMOVE_MOD=hptiop
elif [ ${DRIVER} = "hptmv" ] || [ ${DRIVER} = "hptmv6" ] || [ ${DRIVER} = "rr2310_00" ] || [ ${DRIVER} = "rr174x" ] || [ ${DRIVER} = "rr172x" ]; then
    	REMOVE_MOD=sata_mv
elif [ ${DRIVER} = "rr272x_1x" ] || [ ${DRIVER} = "rr274x_3x" ] || [ ${DRIVER} = "rr276x" ] || [ ${DRIVER} = "rr278x" ]; then
    	REMOVE_MOD=mvsas
elif [ ${DRIVER} = "hpt374" ]; then	
	if ( grep "pata_hpt3x2n" /lib/modules/${KERNEL}/modules.dep -s -q ); then
		REMOVE_MOD=pata_hpt3x2n
	elif ( grep "pata_hpt37x" /lib/modules/${KERNEL}/modules.dep -s -q); then
		REMOVE_MOD=pata_hpt37x
	elif ( grep "/hpt366" /lib/modules/${KERNEL}/modules.dep -s -q ); then
		REMOVE_MOD=hpt366
	fi
fi

if test -d /etc/udev/rules.d; then
	cp $CWD/60-persistent-storage-hptblock.rules /etc/udev/rules.d/
fi

mkdir /tmp/modules
(cd /tmp/modules; gzip -dc $CWD/modules.cgz | cpio -idum) 1>/dev/null 2>&1 || { echo "Fail to extract drivers!"; exit 1; }

if test -d /usr/lib/dracut/modules.d; then
	 if test -f /usr/lib/dracut/modules.d/40hptdrv/dracut-hptdrv.sh; then
		if ! grep -s -q "^modprobe $DRIVER" /usr/lib/dracut/modules.d/40hptdrv/dracut-hptdrv.sh; then
			echo "modprobe $DRIVER 2> /dev/null" >> /usr/lib/dracut/modules.d/40hptdrv/dracut-hptdrv.sh
		fi
	else
		mkdir -p /usr/lib/dracut/modules.d/40hptdrv
		cp $CWD/60-persistent-storage-hptblock.rules /usr/lib/dracut/modules.d/40hptdrv/
		cp $CWD/dracut-hptdrv.sh /usr/lib/dracut/modules.d/40hptdrv/
		cp $CWD/module-setup.sh /usr/lib/dracut/modules.d/40hptdrv/
		chmod +x /usr/lib/dracut/modules.d/40hptdrv/dracut-hptdrv.sh
		chmod +x /usr/lib/dracut/modules.d/40hptdrv/module-setup.sh
	fi
fi

for ver in /lib/modules/[234].*; do
	kver=`basename $ver`
	[ -d /lib/modules/$kver/kernel/fs ] || continue
	mod=`find /lib/modules/$kver/kernel/fs -type f -name *.${KMOD}.xz 2> /dev/null | head -1`
	vermagic=`modinfo -F vermagic $mod 2> /dev/null`
	[ "$vermagic" = "" ] && vermagic=`strings $mod | grep vermagic`
	( echo "$vermagic" | grep " 386 " -s -q ) && MACHINE=i386
	( echo "$vermagic" | grep " 586 " -s -q ) && MACHINE=i586
	( echo "$vermagic" | grep " 686 " -s -q ) && MACHINE=i686
	( echo "$vermagic" | grep " x86_64 " -s -q ) && MACHINE=x86_64

	if [ -d /lib/modules/${kver} -a -d /tmp/modules/${kver}/${MACHINE}/ ]; then
		echo "Updating ${kver}..."
                REMOVE_MOD_FILE=`grep ${REMOVE_MOD} /lib/modules/${kver}/modules.dep 2>/dev/null |cut -d\: -f1`
		if [ "${REMOVE_MOD_FILE}" != "" ];then
			rm -f ${REMOVE_MOD_FILE} /lib/modules/${kver}/${REMOVE_MOD_FILE}
		fi
		mv /boot/${INITRD}-${kver}.img /boot/${INITRD}-${kver}.img.save 
		cp /tmp/modules/${kver}/${MACHINE}/${DRIVER}.${KMOD} /lib/modules/${kver}/kernel/drivers/scsi/ &&
		/sbin/depmod -a ${kver}

		if [ $? = 1 ]; then
			echo "Postinstall 1 failed! Please reinstall your GNU/Linux system."
			exit
		fi

		( grep -s -q scsi_mod /lib/modules/${kver}/modules.dep ) && PRESCSI="--preload scsi_mod"
		( grep -s -q sd_mod /lib/modules/${kver}/modules.dep ) && PRESD="--preload sd_mod"
		( grep -s -q scsi_wait_scan /lib/modules/${kver}/modules.dep ) && WITHWAIT="--with=scsi_wait_scan"

		# Use dracut(From Fedora 12) to generate initrd if possible.
		if [ -x /${BIN}/dracut ]; then
			if [ ${FIRSTK} = 4 ]; then
				dracut -f
			else
				/${BIN}/dracut --add-drivers ${DRIVER} -f /boot/${INITRD}-${kver}.img ${kver} 2> /dev/null
			fi
		elif [ -x /sbin/mkinitrd ]; then
			if [ ${DRIVER} = ${REMOVE_MOD} ]; then
				/sbin/mkinitrd ${PRESCSI} ${PRESD} --with=${DRIVER} ${WITHWAIT} /boot/${INITRD}-${kver}.img ${kver}
			else
				/sbin/mkinitrd ${PRESCSI} ${PRESD} --with=${DRIVER} ${WITHWAIT} --builtin=${REMOVE_MOD} /boot/${INITRD}-${kver}.img ${kver}
			fi
		else
			echo "Postinstall 2 failed! No tool to generate initrd image."
			exit
		fi

		if [ $? = 1 ]; then
			echo "Postinstall 2 failed! Please reinstall your GNU/Linux system."
			exit
		fi
	fi
done

rm -fr /tmp/modules

cp /etc/fstab{,.orig}

cat /etc/fstab.orig | grep hptblock | cut -d\  -f1 | while read line; do
	uuid=`blkid -o export $line | grep ^UUID=`
	if [ "$uuid" != "" ]; then
		sed -i s#"$line\ "#"$uuid\ "# /etc/fstab
	fi
done 

echo "Driver installation step 2 completed."
